@@ -220,7 +220,7 @@ perfsetup(struct perfinfo* info)
 #endif
 		signal(SIGTERM, perf_sigh) == SIG_ERR)
 		fatal_exit("could not bind to signal");
-	info->io = (struct perfio*)calloc(sizeof(struct perfio), info->io_num);
+	info->io = (struct perfio*)calloc(info->io_num, sizeof(struct perfio));
 	if(!info->io) fatal_exit("out of memory");
 #ifndef S_SPLINT_S
 	FD_ZERO(&info->rset);
@@ -501,8 +501,8 @@ qlist_grow_capacity(struct perfinfo* info)
 {
 	size_t newcap = (size_t)((info->qlist_capacity==0)?16:
 		info->qlist_capacity*2);
-	uint8_t** d = (uint8_t**)calloc(sizeof(uint8_t*), newcap);
-	size_t* l = (size_t*)calloc(sizeof(size_t), newcap);
+	uint8_t** d = (uint8_t**)calloc(newcap, sizeof(uint8_t*));
+	size_t* l = (size_t*)calloc(newcap, sizeof(size_t));
 	if(!d || !l) fatal_exit("out of memory");
 	if(info->qlist_data && info->qlist_capacity)
 		memcpy(d, info->qlist_data, sizeof(uint8_t*)*
